@@ -82,7 +82,9 @@ NULL
 #' \enumerate{
 #'
 #' \item Application dependencies are computed by examining the \R code
-#' throughout the project for \code{library} and \code{require} calls.
+#' throughout the project for \code{library} and \code{require} calls. You can
+#' opt out of this behavior by setting \code{infer.dependencies} to
+#' \code{FALSE}.
 #'
 #' \item A snapshot is taken of the version of each package currently used by
 #' the project as described in \code{\link{snapshot}}, and each package's
@@ -110,6 +112,8 @@ NULL
 #'   \code{\link{packrat-options}}.
 #' @param enter Boolean, enter packrat mode for this project after finishing a init?
 #' @param restart If \code{TRUE}, restart the R session after init.
+#' @param infer.dependencies If \code{TRUE}, infer package dependencies by
+#'   examining the \R code.
 #'
 #' @note
 #'
@@ -129,8 +133,54 @@ NULL
 init <- function(project = '.',
                  options = NULL,
                  enter = TRUE,
-                 restart = enter) {
+                 restart = enter,
+                 infer.dependencies = TRUE)
+{
+  ## Get the initial directory structure, so we can rewind if necessary
+  project <- normalizePath(project, winslash = '/', mustWork = TRUE)
+  message("Initializing packrat project in directory:\n- ", surround(prettyDir(project), "\""))
 
+  ## A set of files that packrat might generate as part of init -- we
+  ## enumerate them here to assist with later cleanup
+  prFiles <- c(
+    file.path(project, ".gitignore"),
+    file.path(project, ".Rprofile"),
+    file.path(project, "packrat"),
+    file.path(project, "packrat", "lib"),
+    file.path(project, "packrat", "lib-R"),
+    file.path(project, "packrat", "src"),
+    file.path(project, "packrat", "packrat.lock"),
+    file.path(project, "packrat", "packrat.opts")
+  )
+
+  priorStructure <- setNames(
+    file.exists(prFiles),
+    prFiles
+  )
+
+  withCallingHandlers(
+    initImpl(project, options, enter, restart, infer.dependencies),
+    error = function(e) {
+      # Undo any changes to the directory that did not exist previously
+      for (i in seq_along(priorStructure)) {
+        file <- names(priorStructure)[[i]]
+        fileExistedBefore <- priorStructure[[i]]
+        fileExistsNow <- file.exists(file)
+        if (!fileExistedBefore && fileExistsNow) {
+          unlink(file, recursive = TRUE)
+        }
+      }
+    }
+  )
+
+}
+
+initImpl <- function(project = getwd(),
+                     options = NULL,
+                     enter = TRUE,
+                     restart = enter,
+                     infer.dependencies = TRUE)
+{
   opts <- get_opts(project = project)
   if (is.null(opts))
     opts <- default_opts()
@@ -158,100 +208,53 @@ init <- function(project = '.',
     }
   }
 
-  ## Get the initial directory structure, so we can rewind if necessary
-  project <- normalizePath(project,
-                           winslash = '/',
-                           mustWork = TRUE)
-  message("Initializing packrat project in directory:\n- ", surround(prettyDir(project), "\""))
+  # Force packrat mode off
+  if (isPackratModeOn())
+    off()
 
-  ## A set of files that packrat might generate as part of init -- we
-  ## enumerate them here to assist with later cleanup
-  prFiles <- c(
-    file.path(project, ".gitignore"),
-    file.path(project, ".Rprofile"),
-    file.path(project, "packrat"),
-    file.path(project, "packrat", "lib"),
-    file.path(project, "packrat", "lib-R"),
-    file.path(project, "packrat", "src"),
-    file.path(project, "packrat", "packrat.lock"),
-    file.path(project, "packrat", "packrat.opts")
+  # We always re-packify so that the current version of packrat present can
+  # insert the appropriate auto-loaders
+  packify(project = project, quiet = TRUE)
+
+  # Make sure the .Rprofile is up to date
+  augmentRprofile(project)
+  options <- initOptions(project, opts) ## writes out packrat.opts and returns generated list
+
+  # Take a snapshot
+  snapshotImpl(project,
+               lib.loc = NULL,
+               ignore.stale = TRUE,
+               fallback.ok = TRUE,
+               infer.dependencies = infer.dependencies)
+
+  # Use the lockfile to copy sources and install packages to the library
+  restore(project, overwrite.dirty = TRUE, restart = FALSE)
+
+  # Copy init.R so a user can 'start from zero' with a project
+  file.copy(
+    instInitFilePath(),
+    file.path(project, "packrat", "init.R")
   )
 
-  priorStructure <- setNames(
-    file.exists(prFiles),
-    prFiles
-  )
+  # Update project settings -- this also involves updating the .gitignore,
+  # etc
+  updateSettings(project, options)
 
-  withCallingHandlers(
+  ## Symlink system libraries always
+  symlinkSystemPackages(project = project)
 
-    expr = {
+  message("Initialization complete!")
 
-      ## Force packrat mode off
-      if (isPackratModeOn())
-        off()
+  if (enter) {
 
-      ## We always re-packify so that the current version of packrat present can
-      ## insert the appropriate auto-loaders
-      packify(project = project, quiet = TRUE)
+    setwd(project)
 
-      ## Make sure the .Rprofile is up to date
-      augmentRprofile(project)
-      options <- initOptions(project, opts) ## writes out packrat.opts and returns generated list
-
-      # Take a snapshot
-      snapshotImpl(project,
-                   available.packages(contrib.url(activeRepos(project))),
-                   lib.loc = NULL,
-                   ignore.stale = TRUE,
-                   fallback.ok = TRUE)
-
-      # Use the lockfile to copy sources and install packages to the library
-      restore(project, overwrite.dirty = TRUE, restart = FALSE)
-
-      # Copy init.R so a user can 'start from zero' with a project
-      file.copy(
-        instInitFilePath(),
-        file.path(project, "packrat", "init.R")
-      )
-
-      # Update project settings -- this also involves updating the .gitignore,
-      # etc
-      updateSettings(project, options)
-
-      ## Symlink system libraries always
-      symlinkSystemPackages(project = project)
-
-      message("Initialization complete!")
-
-      if (enter) {
-
-        setwd(project)
-
-        # Restart R if the environment is capable of it (otherwise enter packrat mode)
-        if (!restart || !attemptRestart())
-          on(project = project, clean.search.path = TRUE)
-      }
-
-      invisible()
-
-    }, ## expr
-
-    error = function(e) {
-
-      # Undo any changes to the directory that did not exist previously
-      for (i in seq_along(priorStructure)) {
-        file <- names(priorStructure)[[i]]
-        fileExistedBefore <- priorStructure[[i]]
-        fileExistsNow <- file.exists(file)
-        if (!fileExistedBefore && fileExistsNow) {
-          unlink(file, recursive = TRUE)
-        }
-      }
-
-    } ## error
-
-  )
+    # Restart R if the environment is capable of it (otherwise enter packrat mode)
+    if (!restart || !attemptRestart())
+      on(project = project, clean.search.path = TRUE)
+  }
 
+  invisible()
 }
 
 #' Apply the most recent snapshot to the library
@@ -397,7 +400,7 @@ restore <- function(project = NULL,
   options(repos = repos)
   on.exit({
     options(repos = externalRepos)
-    }, add = TRUE)
+  }, add = TRUE)
 
   # Install each package from CRAN or github, from binaries when available and
   # then from sources.
@@ -593,4 +596,3 @@ lockInfo <- function(project, property='packages', fatal=TRUE) {
   }
   readLockFile(lockFilePath)[[property]]
 }
-
